//--------------------------------------------------
// DirectoryListView.cs (c) 2005 by Charles Petzold
//--------------------------------------------------
using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

class DirectoryListView : ListView
{
    string strDirectory;

    public DirectoryListView()
    {
        // ListView-Spalten erzeugen.
        Columns.Add("Name", 150, HorizontalAlignment.Left);
        Columns.Add("Gre", 100, HorizontalAlignment.Right);
        Columns.Add("Typ", 100, HorizontalAlignment.Left);
        Columns.Add("nderungsdatum", 150, HorizontalAlignment.Left);

        // ListView-Bildlisten erzeugen.
        SmallImageList = new ImageList();
        LargeImageList = new ImageList();
        LargeImageList.ImageSize = new Size(32, 32);
    }

    public string Directory
    {
        get { return strDirectory; }
        set
        {
            // Alle Elemente und die Bildlisten lschen.
            Items.Clear();
            SmallImageList.Images.Clear();
            LargeImageList.Images.Clear();

            // Ordnersymbol laden.
            Icon icn = new Icon(GetType(), "Resource.Folder.ico");
            SmallImageList.Images.Add(icn);
            LargeImageList.Images.Add(icn);

            // DirectoryInfo-Objekt auf Basis des angeforderten 
            // Verzeichnisses erstellen.
            DirectoryInfo dirinfo = new DirectoryInfo(strDirectory = value);

            // Alle Unterverzeichnisse holen und zum ListView hinzufgen.
            foreach (DirectoryInfo dir in dirinfo.GetDirectories())
            {
                ListViewItem item = new ListViewItem(dir.Name);
                item.ImageIndex = 0;
                item.Tag = "dir";
                item.SubItems.Add("");
                item.SubItems.Add("Dateiordner");
                item.SubItems.Add(dir.LastAccessTime.ToString());

                Items.Add(item);
            }

            int iImage = 1;

            // Alle Dateien holen und zum ListView hinzufgen.
            foreach (FileInfo file in dirinfo.GetFiles())
            {
                ListViewItem item = new ListViewItem(file.Name);

                icn = Icon.ExtractAssociatedIcon(
                                Path.Combine(file.DirectoryName, file.Name));

                SmallImageList.Images.Add(icn);
                LargeImageList.Images.Add(icn);
                item.ImageIndex = iImage++;

                item.SubItems.Add(file.Length.ToString("N0"));
                item.SubItems.Add(
                    Path.GetExtension(file.Name).ToUpper() == ".EXE" ?
                        "Programm" : "Dokument");
                item.SubItems.Add(file.LastWriteTime.ToString());

                Items.Add(item);
            }
        }
    }
    protected override void OnMouseDown(MouseEventArgs args)
    {
        base.OnMouseDown(args);

        // Ansicht bei Rechtsklick ndern.
        if (args.Button == MouseButtons.Right)
            View = (View)(((int)View + 1) % 5);
    }
    protected override void OnItemActivate(EventArgs args)
    {
        base.OnItemActivate(args);

        // Falls ein Verzeichnis angeklickt wurde, in dieses Verzeichnis wechseln.
        if ((string)SelectedItems[0].Tag == "dir")
        {
            Directory = Path.Combine(Directory, SelectedItems[0].Text);
        }
        else
        {
            // Andernfalls die Programme starten, die mit der Datei bzw. den 
            // Dateien verknpft sind.
            foreach (ListViewItem item in SelectedItems)
            {
                try
                {
                    Process.Start(Path.Combine(Directory, item.Text));
                }
                catch
                {
                }
            }
        }
    }
}