//-------------------------------------------------
// ColorScrollTable.cs (c) 2005 by Charles Petzold
//-------------------------------------------------
using System;
using System.Drawing;
using System.Windows.Forms;

class ColorScrollTable : Form
{
    Panel pnlColor;
    Label[] alblValue = new Label[3];

    [STAThread]
    public static void Main()
    {
        Application.EnableVisualStyles();
        Application.Run(new ColorScrollTable());
    }
    public ColorScrollTable()
    {
        Text = "Color Scroll mit TableLayoutPanel";

        // SplitContainer anlegen, der den Clientbereich fllt.
        SplitContainer splt = new SplitContainer();
        splt.Parent = this;
        splt.Dock = DockStyle.Fill;
        splt.SplitterDistance = ClientSize.Width / 2;

        // Das TableLayoutPanel kommt links neben den Splitter.
        TableLayoutPanel table = new TableLayoutPanel();
        table.Parent = splt.Panel1;
        table.Dock = DockStyle.Fill;
        table.BackColor = Color.White;
        table.ColumnCount = 3;
        table.RowCount = 3;

        // Den rechten SplitterPanel-Bereich als Feld speichern.
        pnlColor = splt.Panel2;

        // Zwei Arrays fr Farbnamen und Anfangswerte.
        string[] astrColors = { "Rot", "Grn", "Blau" };
        int[] aiPanelColor = new int[3] { pnlColor.BackColor.R, 
                                          pnlColor.BackColor.G, 
                                          pnlColor.BackColor.B };

        // Die drei Spalten (rot, grn, blau) in einer Schleife durchgehen.
        for (int col = 0; col < 3; col++)
        {
            // Die Beschriftungen oben geben die Farbe an.
            Label lbl = new Label();
            lbl.AutoSize = true;
            lbl.Anchor = AnchorStyles.None;
            lbl.Text = astrColors[col];
            lbl.ForeColor = Color.FromName(astrColors[col]);
            table.Controls.Add(lbl, col, 0);

            // Scrollbar zum Einstellen neuer Werte.
            VScrollBar vscrl = new VScrollBar();
            vscrl.Parent = table;
            vscrl.Anchor = AnchorStyles.Top | AnchorStyles.Bottom;
            vscrl.TabStop = true;
            vscrl.LargeChange = 16;
            vscrl.Maximum = 255 + vscrl.LargeChange - 1;
            vscrl.Value = aiPanelColor[col];
            vscrl.ValueChanged += OnScrollValueChanged;
            table.Controls.Add(vscrl, col, 1);

            // Das Beschriftungsfeld fr den Farbwert wird an den Scrollbar gebunden.
            alblValue[col] = new Label();
            alblValue[col].AutoSize = true;
            alblValue[col].Anchor = AnchorStyles.None;
            alblValue[col].ForeColor = Color.FromName(astrColors[col]);
            alblValue[col].DataBindings.Add("Text", vscrl, "Value");
            table.Controls.Add(alblValue[col], col, 2);

            // ber ColumnStyles drei gleich breite Spalten auswhlen.
            table.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 33));
        }

        // RowStyles legt fest, dass die mittlere Zeile so gro wie mglich ist.
        table.RowStyles.Add(new RowStyle(SizeType.AutoSize));
        table.RowStyles.Add(new RowStyle(SizeType.Percent, 100));
        table.RowStyles.Add(new RowStyle(SizeType.AutoSize));
    }
    // Wenn sich der Wert eines Scrollbars ndert, die Hintergrundfarbe des 
    // Bereichs ndern.
    void OnScrollValueChanged(object objSrc, EventArgs args)
    {
        pnlColor.BackColor = Color.FromArgb(int.Parse(alblValue[0].Text), 
                                            int.Parse(alblValue[1].Text), 
                                            int.Parse(alblValue[2].Text));
    }
}