//-----------------------------------------------
// DocumentRuler1.cs (c) 2005 by Charles Petzold
//-----------------------------------------------
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

public partial class DocumentRuler : Control
{
    // Private Felder.
    int iLeftMargin;
    float fDpi;
    Control ctrlDocument;

    // RulerSlider-Objekte.
    LeftIndent rsLeftIndent = new LeftIndent();
    RightIndent rsRightIndent = new RightIndent();
    FirstLineIndent rsFirstIndent = new FirstLineIndent();
    TextWidth rsTextWidth = new TextWidth();
    List<RulerSlider> rsCollection = new List<RulerSlider>();

    // Konstruktor.
    public DocumentRuler()
    {
        Dock = DockStyle.Top;
        ResizeRedraw = true;
        TabStop = false;
        Height = 23;
        Font = new Font(Font.Name, 14, GraphicsUnit.Pixel);

        Graphics grfx = CreateGraphics();
        fDpi = grfx.DpiX;
        grfx.Dispose();

        rsCollection.Add(rsLeftIndent);
        rsCollection.Add(rsRightIndent);
        rsCollection.Add(rsFirstIndent);
        rsCollection.Add(rsTextWidth);
    }

    // ffentliche Eigenschaften.
    public float TextWidth
    {
        get { return rsTextWidth.Value; }
        set
        {
            rsTextWidth.Value = value;
            CalculateDisplayOffsets();
        }
    }
    public float LeftIndent
    {
        get { return rsLeftIndent.Value; } 
        set 
        {
            rsLeftIndent.Value = value;
            CalculateDisplayOffsets();
        }
    }
    public float RightIndent
    {
        get { return rsRightIndent.Value; }
        set
        {
            rsRightIndent.Value = value;
            CalculateDisplayOffsets();
        }
    }
    public float FirstLineIndent
    {
        get { return rsFirstIndent.Value; }
        set
        {
            rsFirstIndent.Value = value;
            CalculateDisplayOffsets();
        }
    }
    public float[] Tabs
    {
        get 
        {
            List<float> fTabs = new List<float>();
        
            foreach (RulerSlider rs in rsCollection)
                if (rs is Tab)
                    fTabs.Add(rs.Value);

            // RichTextBox will die Tabstopps numerisch sortiert
            float[] afTabs = fTabs.ToArray();
            Array.Sort(afTabs); 
            return afTabs;
        }            
        set 
        {
            // Zuerst Tabstopps lschen, die nicht im Array sind.
            List<Tab> rsTabsDelete = new List<Tab>();

            foreach (RulerSlider rs in rsCollection)
                if (rs is Tab && (Array.IndexOf(value, rs.Value) == -1))
                    rsTabsDelete.Add(rs as Tab);

            foreach (Tab tab in rsTabsDelete)
            {
                rsCollection.Remove(tab);
                Invalidate(tab.Rectangle);
            }

            // Dann Tabstopps hinzufgen, die nicht in rsCollection sind.
            foreach (float fTab in value)
            {
                bool bAdd = true;

                foreach (RulerSlider rs in rsCollection)
                    if (rs is Tab && rs.Value == fTab)
                        bAdd = false;

                if (bAdd)
                {
                    Tab tab = new Tab();
                    tab.Value = fTab;
                    tab.X = LeftMargin + InchesToPixels(fTab);
                    rsCollection.Add(tab);
                    Invalidate(tab.Rectangle);
                }
            }
        }
    }
    public int LeftMargin
    {
        get { return iLeftMargin; }
        set
        {
            iLeftMargin = value;
            CalculateDisplayOffsets();
        }
    }

    // Anzeigen einer Zeile, wenn die Schieber bewegt wurden.
    public Control DocumentControl
    {
        get { return ctrlDocument; }
        set { ctrlDocument = value; }
    }

    // Diese beiden Methoden berechnen X-Werte fr die vier Schiebertypen
    //  (ohne Tabstopps).  Falls sich die X-Werte ndern, wird das Rechteck
    //  an der vorherigen Position und der neuen Position fr ungltig erklrt.
    void CalculateDisplayOffsets()
    {
        CalculateDisplayOffsets2(rsTextWidth, LeftMargin + 
                        InchesToPixels(rsTextWidth.Value));
        CalculateDisplayOffsets2(rsLeftIndent, LeftMargin + 
                        InchesToPixels(rsLeftIndent.Value));
        CalculateDisplayOffsets2(rsRightIndent, LeftMargin + 
                        InchesToPixels(TextWidth - rsRightIndent.Value));
        CalculateDisplayOffsets2(rsFirstIndent, LeftMargin + 
                        InchesToPixels(LeftIndent + rsFirstIndent.Value));
    }
    void CalculateDisplayOffsets2(RulerSlider rs, int xNew)
    {
        if (rs.X != xNew)
        {
            Invalidate(rs.Rectangle);
            rs.X = xNew;
            Invalidate(rs.Rectangle);
        }
    }
    float PixelsToInches(int i)
    {
        return i / fDpi;
    }
    int InchesToPixels(float f)
    {
        return (int)Math.Round(f * fDpi);
    }
}

