//-----------------------------------------------
// DocumentRuler2.cs (c) 2005 by Charles Petzold
//-----------------------------------------------
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

public partial class DocumentRuler : Control
{
    // ffentliches Ereignis.
    public event RulerEventHandler Change;

    // Private Felder fr das Ziehen mit der Maus.
    RulerSlider rsDragging;
    Point ptDown;
    int xOriginal;
    int xLineOverTextBox;

    // Die Methode OnPaint erledigt fast die ganze Zeichenarbeit.
    protected override void OnPaint(PaintEventArgs args)
    {
        Graphics grfx = args.Graphics;

        Rectangle rect = new Rectangle(LeftMargin, 0, 
                                rsTextWidth.X - LeftMargin, Height - 4);
        grfx.FillRectangle(Brushes.White, rect);
        ControlPaint.DrawBorder3D(grfx, rect);

        for (int i = 1; i < 8 * PixelsToInches(Width); i++)
        {
            int x = LeftMargin + InchesToPixels(i / 8f);

            if (i % 8 == 0)
            {
                StringFormat strfmt = new StringFormat();
                strfmt.Alignment = strfmt.LineAlignment = StringAlignment.Center;
                grfx.DrawString((i / 8).ToString(), Font, 
                                Brushes.Black, x, 9, strfmt);
            }
            else if (i % 4 == 0)
            {
                grfx.DrawLine(Pens.Black, x, 7, x, 10);
            }
            else
            {
                grfx.DrawLine(Pens.Black, x, 8, x, 9);
            }
        }

        // Alle Schieber anzeigen.
        foreach (RulerSlider rs in rsCollection)
            rs.Draw(grfx);

        return;
    }

    // OnMouseDown zum Bewegen der Schieber und zum Anlegen der Tabstopps.
    protected override void OnMouseDown(MouseEventArgs args)
    {
        // Ignorieren, falls es nicht die linke Taste ist.
        if ((args.Button & MouseButtons.Left) == 0)
            return;

        // In einer Schleife ber alle Schieber gehen und feststellen, ob 
        // einer angeklickt wurde.
        foreach (RulerSlider rs in rsCollection)
            if (rs.HitTest(args.Location))
            {
                rsDragging = rs;
                ptDown = args.Location;
                xOriginal = rsDragging.X;

                if (rsDragging is TextWidth)
                    Cursor.Current = Cursors.SizeWE;

                DrawReversibleLine(xLineOverTextBox = args.X);
                return;
            }
        // Falls kein vorhandener Tabstopp angeklickt wurde, neuen erstellen.
        rsDragging = new Tab();
        rsCollection.Add(rsDragging);
        ptDown = args.Location;
        xOriginal = rsDragging.X = ptDown.X;

        Invalidate(rsDragging.Rectangle);
        DrawReversibleLine(xLineOverTextBox = args.X);
        return;
    }

    // OnMouseMove zum Bewegen der Schieber.
    protected override void OnMouseMove(MouseEventArgs args)
    {
        if (!Capture)   // z.B. Maustaste nicht gedrckt.
        {
            // Cursor ndern, falls ber dem Ende von TextWidth.
            if (!rsRightIndent.HitTest(args.Location) && 
                 rsTextWidth.HitTest(args.Location))
                    Cursor.Current = Cursors.SizeWE;
            return;
        }

        // Falls rsDragging ungleich null, sind wir in einer Schiebeaktion.
        if (rsDragging != null)
        {
            if (rsDragging is TextWidth)
                Cursor.Current = Cursors.SizeWE;

            int xNow = xOriginal - ptDown.X + args.X;

            // Schieber drfen nicht ber die Grenzen hinauswandern!
            if (rsDragging is Tab && (xNow < LeftMargin || xNow > rsTextWidth.X))
                return;

            if ((rsDragging == rsLeftIndent || rsDragging == rsFirstIndent) &&
                    (xNow < LeftMargin || xNow > rsRightIndent.X))
                return;

            if (rsDragging == rsRightIndent && (xNow > rsTextWidth.X ||
                    xNow < rsLeftIndent.X || xNow < rsFirstIndent.X))
                return;

            if (rsDragging == rsTextWidth && xNow < rsRightIndent.X)
                return;

            if (rsDragging == rsTextWidth)
            {
                Invalidate(new Rectangle(Math.Min(rsDragging.X, xOriginal) - 1, 0,
                                Math.Abs(rsDragging.X - xOriginal) + 2, Height));
                rsDragging.X = xNow;
            }
            else
            {
                // X-Eigenschaft des Schiebers aktualisieren und 
                // alte und neue Position fr ungltig erklren.
                Invalidate(rsDragging.Rectangle);
                rsDragging.X = xNow;
                Invalidate(rsDragging.Rectangle);
            }
            // Zeile ber Textfeld verschieben.
            DrawReversibleLine(xLineOverTextBox);
            DrawReversibleLine(xLineOverTextBox = args.X);
        }
    }

    // OnMouseUp ist die neue Position des Schiebers.
    protected override void OnMouseUp(MouseEventArgs args)
    {
        if (rsDragging != null)
        {
            // Neue Value-Eigenschaften berechnen und das Ereignis auslsen.
            if (rsDragging == rsLeftIndent || rsDragging == rsFirstIndent)
            {
                rsLeftIndent.Value = PixelsToInches(rsLeftIndent.X - LeftMargin);
                rsFirstIndent.Value = 
                        PixelsToInches(rsFirstIndent.X - rsLeftIndent.X);
                OnChange(new RulerEventArgs(rsDragging.RulerProperty));
            }
            else if (rsDragging == rsRightIndent || rsDragging == rsTextWidth)
            {
                rsTextWidth.Value = PixelsToInches(rsTextWidth.X - LeftMargin);
                rsRightIndent.Value = 
                        PixelsToInches(rsTextWidth.X - rsRightIndent.X);
                OnChange(new RulerEventArgs(rsTextWidth.RulerProperty));
                OnChange(new RulerEventArgs(rsRightIndent.RulerProperty));
            }

            else if (rsDragging is Tab)
            {
                rsDragging.Value = PixelsToInches(rsDragging.X - LeftMargin);
                OnChange(new RulerEventArgs(rsDragging.RulerProperty));
            }
            // Schiebeoperation beenden.
            rsDragging = null;
            DrawReversibleLine(xLineOverTextBox);
        }
    }

    // Linie in Bildschirmkoordinaten durch das Textfeld zeichnen.
    void DrawReversibleLine(int x)
    {
        if (ctrlDocument != null)
        {
            Point pt1 = ctrlDocument.PointToScreen(new Point(x, 0));
            Point pt2 = ctrlDocument.PointToScreen(
                                    new Point(x, ctrlDocument.Height));
            ControlPaint.DrawReversibleLine(pt1, pt2, ctrlDocument.BackColor);
        }
    }

    // Methode OnChange lst das Change-Ereignis aus.
    protected virtual void OnChange(RulerEventArgs args)
    {
        if (Change != null)
            Change(this, args);
    }
}