//----------------------------------------------------
// FormattingToolStrip.cs (c) 2005 by Charles Petzold
//----------------------------------------------------
using System;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;

class FormattingToolStrip : Form
{
    protected RichTextBox txtbox;
    ToolStripComboBox comboName, comboSize;
    ToolStripButton btnBold, btnItalic, btnUnderline, btnStrikeout;
    ToolStripButton btnLeft, btnRight, btnCenter, btnBullets;
    ToolStripTextBox txtLeftIndent, txtRightIndent, txtFirstLine;
    ColorDialog clrdlg = new ColorDialog();
    float xDpi;
    bool bSuspendSelectionChanged = false;

    [STAThread]
    public static void Main()
    {
        Application.EnableVisualStyles();
        Application.Run(new FormattingToolStrip());
    }
    public FormattingToolStrip()
    {
        Text = "Formatierungs-ToolStrip";
        Width = 800;

        // Horizontale Bildschirmauflsung in dpi abfragen.
        Graphics grfx = CreateGraphics();
        xDpi = grfx.DpiX;
        grfx.Dispose();

        // Viele Bitmaps fr die Symbolleiste laden.
        ImageList imglst = new ImageList();
        imglst.TransparentColor = Color.Magenta;
        imglst.Images.Add("BackColor", new Bitmap(GetType(), "ChooseColor.bmp"));
        imglst.Images.Add("ForeColor", new Bitmap(GetType(), "Forecolor.bmp"));
        imglst.Images.Add("Left", 
            new Bitmap(GetType(), "AlignTableCellMiddleLeftJustHS.bmp"));
        imglst.Images.Add("Right", 
            new Bitmap(GetType(), "AlignTableCellMiddleRight.bmp"));
        imglst.Images.Add("Center", 
            new Bitmap(GetType(), "AlignTableCellMiddleCenter.bmp"));
        imglst.Images.Add("Bullets", new Bitmap(GetType(), "List_Bullets.bmp"));

        // Einige weitere Bitmaps erstellen.
        imglst.Images.Add("Bold", FontStyleBitmap("F", FontStyle.Bold));
        imglst.Images.Add("Italic", FontStyleBitmap("K", FontStyle.Italic));
        imglst.Images.Add("Underline", FontStyleBitmap("U", FontStyle.Underline));
        imglst.Images.Add("Strikeout", FontStyleBitmap("D", FontStyle.Strikeout));

        // Das RichTextBox-Steuerelement im Zentrum des Clientbereichs anlegen.
        txtbox = new RichTextBox();
        txtbox.Parent = this;
        txtbox.Dock = DockStyle.Fill;
        txtbox.SelectionChanged += TextBoxOnSelectionChanged;

        // ToolStrip-Steuerelement anlegen.
        ToolStrip tool = new ToolStrip();
        tool.Parent = this;
        tool.ImageList = imglst;

        // Kombinationsfeld fr Schriftart anlegen und fllen.
        comboName = new ToolStripComboBox();
        comboName.ToolTipText = "Schriftart";
        comboName.SelectedIndexChanged += NameOnSelectionChanged;
        tool.Items.Add(comboName);

        foreach (FontFamily fntfam in FontFamily.Families)
            comboName.Items.Add(fntfam.Name);

        // Kombinationsfeld fr Schriftgrad anlegen und fllen.
        comboSize = new ToolStripComboBox();
        comboSize.ToolTipText = "Schriftgrad";
        comboSize.SelectedIndexChanged += SizeOnSelectionChanged;
        tool.Items.Add(comboSize);

        for (int i = 8; i <= 10; i++)
            comboSize.Items.Add(i.ToString());
        for (int i = 12; i <= 28; i += 2)
            comboSize.Items.Add(i.ToString());
        for (int i = 36; i <= 72; i += 12)
            comboSize.Items.Add(i.ToString());

        // Schaltflchen fr Fett, Kursiv, Unterstrichen und 
        // Durchgestrichen erstellen.
        btnBold = new ToolStripButton();
        btnBold.ImageKey = "Bold";
        btnBold.ToolTipText = "Fett";
        btnBold.Tag = FontStyle.Bold;
        btnBold.CheckOnClick = true;
        btnBold.Click += FontStyleOnClick;
        tool.Items.Add(btnBold);

        btnItalic = new ToolStripButton();
        btnItalic.ImageKey = "Italic";
        btnItalic.ToolTipText = "Kursiv";
        btnItalic.Tag = FontStyle.Italic;
        btnItalic.CheckOnClick = true;
        btnItalic.Click += FontStyleOnClick;
        tool.Items.Add(btnItalic);

        btnUnderline = new ToolStripButton();
        btnUnderline.ImageKey = "Underline";
        btnUnderline.ToolTipText = "Unterstrichen";
        btnUnderline.Tag = FontStyle.Underline;
        btnUnderline.CheckOnClick = true;
        btnUnderline.Click += FontStyleOnClick;
        tool.Items.Add(btnUnderline);

        btnStrikeout = new ToolStripButton();
        btnStrikeout.ImageKey = "Strikeout";
        btnStrikeout.ToolTipText = "Durchgestrichen";
        btnStrikeout.CheckOnClick = true;
        btnStrikeout.Tag = FontStyle.Strikeout;
        btnStrikeout.Click += FontStyleOnClick; 
        tool.Items.Add(btnStrikeout);
        tool.Items.Add(new ToolStripSeparator());

        // Dropdownschaltflche fr Hintergrundfarbe erstellen.
        ToolStripSplitButton spltbtn = new ToolStripSplitButton();
        spltbtn.ImageKey = "BackColor";
        spltbtn.ToolTipText = "Hintergrundfarbe";
        spltbtn.ButtonClick += delegate 
        { 
            txtbox.SelectionBackColor = txtbox.BackColor; 
        };
        spltbtn.DropDownOpening += ColorOnDropDownOpening;
        tool.Items.Add(spltbtn);

        // PropertyInfo fr Hintergrundfarbe erstellen.
        PropertyInfo pi = typeof(RichTextBox).GetProperty("SelectionBackColor");

        // Farbauswahl-Steuerelement zum Dropdown hinzufgen.
        ToolStripColorGrid clrgrid = new ToolStripColorGrid();
        clrgrid.Name = "ColorGrid";
        clrgrid.Tag = pi;
        clrgrid.Click += ColorGridOnClick;
        spltbtn.DropDownItems.Add(clrgrid);
        spltbtn.DropDownItems.Add(new ToolStripSeparator());

        // Element "Weitere Farben" zum Dropdown hinzufgen.
        ToolStripMenuItem item = new ToolStripMenuItem("Weitere Farben...");
        item.Tag = pi;
        item.Click += MoreColorsOnClick;
        spltbtn.DropDownItems.Add(item);

        // Genauso die Schaltflche fr die Vordergrundfarbe erstellen.
        spltbtn = new ToolStripSplitButton();
        spltbtn.ImageKey = "ForeColor";
        spltbtn.ToolTipText = "Vordergrundfarbe";
        spltbtn.ButtonClick += delegate
        {
            txtbox.SelectionColor = txtbox.ForeColor;
        };
        spltbtn.DropDownOpening += ColorOnDropDownOpening;
        tool.Items.Add(spltbtn);

        // PropertyInfo fr Vordergrundfarbe erstellen.
        pi = typeof(RichTextBox).GetProperty("SelectionColor");

        // Farbauswahl-Steuerelement und "Weitere Farben"-Element.
        clrgrid = new ToolStripColorGrid();
        clrgrid.Name = "ColorGrid";
        clrgrid.Tag = pi;
        clrgrid.Click += ColorGridOnClick;
        spltbtn.DropDownItems.Add(clrgrid);
        spltbtn.DropDownItems.Add(new ToolStripSeparator());

        item = new ToolStripMenuItem("Weitere Farben...");
        item.Tag = pi;
        item.Click += MoreColorsOnClick;
        spltbtn.DropDownItems.Add(item);

        tool.Items.Add(new ToolStripSeparator());
        
        // Schaltflchen fr Ausrichtung Links, Rechts und Zentriert erstellen.
        btnLeft = new ToolStripButton();
        btnLeft.ImageKey = "Left";
        btnLeft.ToolTipText = "Ausrichtung Links";
        btnLeft.Tag = HorizontalAlignment.Left;
        btnLeft.Checked = true;
        btnLeft.Click += AlignOnClick;
        tool.Items.Add(btnLeft);

        btnRight = new ToolStripButton();
        btnRight.ImageKey = "Right";
        btnRight.ToolTipText = "Ausrichtung Rechts";
        btnRight.Tag = HorizontalAlignment.Right;
        btnRight.Click += AlignOnClick;
        tool.Items.Add(btnRight);

        btnCenter = new ToolStripButton();
        btnCenter.ImageKey = "Center";
        btnCenter.ToolTipText = "Ausrichtung Zentriert";
        btnCenter.Tag = HorizontalAlignment.Center;
        btnCenter.Click += AlignOnClick;
        tool.Items.Add(btnCenter);

        // Schaltflche fr Aufzhlungszeichen erstellen.
        btnBullets = new ToolStripButton();
        btnBullets.ImageKey = "Bullets";
        btnBullets.ToolTipText = "Aufzhlungszeichen";
        btnBullets.CheckOnClick = true;
        btnBullets.Click += BulletsOnClick;
        tool.Items.Add(btnBullets);
        tool.Items.Add(new ToolStripSeparator());

        // Beschriftungsfelder und Textfelder fr Einzug.
        ToolStripLabel lbl = new ToolStripLabel("Links:");
        tool.Items.Add(lbl);

        txtLeftIndent = new ToolStripTextBox();
        txtLeftIndent.ToolTipText = "Einzug links in Inch";
        txtLeftIndent.TextChanged += IndentOnTextChanged;
        tool.Items.Add(txtLeftIndent);

        lbl = new ToolStripLabel("Rechts:");
        tool.Items.Add(lbl);

        txtRightIndent = new ToolStripTextBox();
        txtRightIndent.ToolTipText = "Einzug rechts in Inch";
        txtRightIndent.TextChanged += IndentOnTextChanged;
        tool.Items.Add(txtRightIndent);

        lbl = new ToolStripLabel("Erste Zeile:");
        tool.Items.Add(lbl);

        txtFirstLine = new ToolStripTextBox();
        txtFirstLine.ToolTipText = "Einzug erste Zeile in Inch";
        txtFirstLine.TextChanged += IndentOnTextChanged;
        tool.Items.Add(txtFirstLine);

        // ToolStrip initialisieren.
        TextBoxOnSelectionChanged(txtbox, EventArgs.Empty);
    }

    // Schaltflchen fr Fett, Kursiv, Unterstrichen und Durchgestrichen
    Bitmap FontStyleBitmap(string str, FontStyle fntstyle)
    {
        Bitmap bm = new Bitmap(16, 16);
        Font fnt = new Font("Times New Roman", 14, fntstyle, GraphicsUnit.Pixel);
        StringFormat strfmt = new StringFormat();
        strfmt.Alignment = StringAlignment.Center;

        Graphics grfx = Graphics.FromImage(bm);
        grfx.DrawString(str, fnt, Brushes.Black, 8, 0, strfmt);
        grfx.Dispose();
        return bm;
    }

    // Immer wenn sich die Auswahl der RichTextBox ndert, werden die 
    // ToolStrip-Elemente angepasst.
    void TextBoxOnSelectionChanged(object objSrc, EventArgs args)
    {
        if (bSuspendSelectionChanged)
            return;

        Font fnt = txtbox.SelectionFont;

        if (fnt != null)
        {
            comboName.SelectedItem = fnt.Name;
            comboSize.Text = fnt.Size.ToString();

            btnBold.Checked = (fnt.Style & FontStyle.Bold) != 0;
            btnItalic.Checked = (fnt.Style & FontStyle.Italic) != 0;
            btnUnderline.Checked = (fnt.Style & FontStyle.Underline) != 0;
            btnStrikeout.Checked = (fnt.Style & FontStyle.Strikeout) != 0;
        }
        else
        {
            comboName.SelectedItem = null;
            comboSize.SelectedItem = null;

            btnBold.CheckState = CheckState.Unchecked;
            btnItalic.CheckState = CheckState.Unchecked;
            btnUnderline.CheckState = CheckState.Unchecked;
            btnStrikeout.CheckState = CheckState.Unchecked;
        }

        HorizontalAlignment hAlign = txtbox.SelectionAlignment;
        btnLeft.Checked = hAlign == HorizontalAlignment.Left;
        btnRight.Checked = hAlign == HorizontalAlignment.Right;
        btnCenter.Checked = hAlign == HorizontalAlignment.Center;
        btnBullets.Checked = txtbox.SelectionBullet;

        txtLeftIndent.Text = ((txtbox.SelectionIndent + 
                        txtbox.SelectionHangingIndent) / xDpi).ToString();
        txtRightIndent.Text = (txtbox.SelectionRightIndent / xDpi).ToString();
        txtFirstLine.Text = (-txtbox.SelectionHangingIndent / xDpi).ToString();
    }

    // Name der Schriftart ndern.
    void NameOnSelectionChanged(object objSrc, EventArgs args)
    {
        ChangeFont(comboName.Text, 0, 0, false);
    }

    // Schriftgrad ndern.
    void SizeOnSelectionChanged(object objSrc, EventArgs args)
    {
        float fSize = float.Parse(comboSize.Text);
        ChangeFont(null, fSize, 0, false);
    }

    // Schriftschnitt ndern.
    void FontStyleOnClick(object objSrc, EventArgs args)
    {
        ToolStripButton btn = (ToolStripButton)objSrc;
        FontStyle fntstyle = (FontStyle)btn.Tag;
        ChangeFont(null, 0, (FontStyle)btn.Tag, btn.Checked);
    }

    // Zentrale Methode zum ndern der Schrift.
    void ChangeFont(string strName, float fSize, FontStyle fntsty, bool bAdd)
    {
        bSuspendSelectionChanged = true;

        int iSelStart = txtbox.SelectionStart;
        int iSelLength = txtbox.SelectionLength;

        for (int iStart1 = iSelStart; iStart1 < iSelStart + iSelLength; )
        {
            txtbox.Select(iStart1, 1);
            Font fnt = txtbox.SelectionFont;

            for (int iStart2 = iStart1 + 1; iStart2 <= iSelStart + iSelLength; iStart2++)
            {
                txtbox.Select(iStart2, 1);
                Font fntNext = txtbox.SelectionFont;

                if (iStart2 == iSelStart + iSelLength || !fnt.Equals(fntNext))
                {
                    txtbox.Select(iStart1, iStart2 - iStart1);

                    if (strName != null)
                        txtbox.SelectionFont = new Font(strName, fnt.Size, fnt.Style);
                    else if (fSize != 0)
                        txtbox.SelectionFont = new Font(fnt.Name, fSize, fnt.Style);
                    else if (bAdd)
                        txtbox.SelectionFont = new Font(fnt, fnt.Style | fntsty);
                    else
                        txtbox.SelectionFont = new Font(fnt, fnt.Style & ~fntsty);

                    iStart1 = iStart2;
                    break;
                }
            }
        }
        bSuspendSelectionChanged = false;
        txtbox.Select(iSelStart, iSelLength);
    }

    // Farbauswahl-Steuerelement initialisieren, wenn sich das Dropdown ffnet.
    void ColorOnDropDownOpening(object objSrc, EventArgs args)
    {
        ToolStripSplitButton btn = (ToolStripSplitButton)objSrc;
        ToolStripColorGrid clrgrid = (ToolStripColorGrid)btn.DropDownItems["ColorGrid"];
        PropertyInfo pi = (PropertyInfo) clrgrid.Tag;
        clrgrid.SelectedColor = (Color)pi.GetValue(txtbox, null);
    }

    // Neue Farbe aus dem Farbauswahl-Steuerelement abrufen.
    void ColorGridOnClick(object objSrc, EventArgs args)
    {
        ToolStripColorGrid clrgrid = (ToolStripColorGrid)objSrc;
        PropertyInfo pi = (PropertyInfo)clrgrid.Tag;
        pi.SetValue(txtbox, clrgrid.SelectedColor, null);
    }

    // Standarddialogfeld Farben anzeigen.
    void MoreColorsOnClick(object objSrc, EventArgs args)
    {
        ToolStripMenuItem item = (ToolStripMenuItem)objSrc;
        PropertyInfo pi = (PropertyInfo)item.Tag;
        clrdlg.Color = (Color)pi.GetValue(txtbox, null);

        if (clrdlg.ShowDialog() == DialogResult.OK)
            pi.SetValue(txtbox, clrdlg.Color, null);
    }

    // Ausrichtung entsprechend der gedrckten Schaltflche ndern.
    void AlignOnClick(object objSrc, EventArgs args)
    {
        ToolStripButton btn = (ToolStripButton)objSrc;
        btnLeft.Checked = btnRight.Checked = btnCenter.Checked = false;
        btn.Checked = true;
        txtbox.SelectionAlignment = (HorizontalAlignment) btn.Tag;
    }

    // Aufzhlungszeichen aktivieren oder deaktivieren.
    void BulletsOnClick(object objSrc, EventArgs args)
    {
        ToolStripButton btn = (ToolStripButton)objSrc;
        txtbox.SelectionBullet = btn.Checked;
    }

    // Einzge entsprechend den Werten in den Textfeldern ndern.
    void IndentOnTextChanged(object objSrc, EventArgs args)
    {
        try
        {
            int iLeftIndent = (int)(xDpi * float.Parse(txtLeftIndent.Text));
            int iRightIndent = (int)(xDpi * float.Parse(txtRightIndent.Text));
            int iFirstLine = (int)(xDpi * float.Parse(txtFirstLine.Text));

            txtbox.SelectionIndent = iLeftIndent + iFirstLine;
            txtbox.SelectionHangingIndent = -iFirstLine;
            txtbox.SelectionRightIndent = iRightIndent;
        }
        catch
        {
        }
    }
}