//--------------------------------------------------------
// DataEntryWithNavigation.cs (c) 2005 by Charles Petzold
//--------------------------------------------------------
using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Xml.Serialization;

class DataEntryWithNavigation : Form
{
    const string strFilter = "XML-Personendateien (*.PersonFileXml)|" +
                             "*.PersonFileXml|Alle Dateien (*.*)|*.*";
    XmlSerializer xmlser = new XmlSerializer(typeof(PersonFile));
    BindingSource bindsrc = new BindingSource();

    [STAThread]
    public static void Main()
    {
        Application.EnableVisualStyles();
        Application.Run(new DataEntryWithNavigation());
    }
    public DataEntryWithNavigation()
    {
        Text = "Einfache Dateneingabe mit Navigation";

        // BindingSource initialisieren.
        FileNewOnClick(null, EventArgs.Empty);

        // Bereich anlegen.
        PersonPanel personpnl = new PersonPanel(bindsrc);
        personpnl.Parent = this;
        personpnl.Dock = DockStyle.Fill;

        // Men anlegen.
        MenuStrip menu = new MenuStrip();
        menu.Parent = this;
        ToolStripMenuItem item = (ToolStripMenuItem)menu.Items.Add("&Datei");
        item.DropDownItems.Add("&Neu", null, FileNewOnClick);
        item.DropDownItems.Add("&ffnen...", null, FileOpenOnClick);
        item.DropDownItems.Add("Speichern &unter...", null, FileSaveAsOnClick);

        // BindingNavigator erstellen.
        BindingNavigator bindnav = new BindingNavigator(true);
        bindnav.Parent = this;
        bindnav.Dock = DockStyle.Bottom;
        bindnav.BindingSource = bindsrc;
    }
    void FileNewOnClick(object objSrc, EventArgs args)
    {
        PersonFile persfile = new PersonFile();
        persfile.Persons.Add(new Person());

        bindsrc.DataSource = persfile;
        bindsrc.DataMember = "Persons";
    }
    void FileOpenOnClick(object objSrc, EventArgs args)
    {
        OpenFileDialog dlg = new OpenFileDialog();
        dlg.Filter = strFilter;

        if (dlg.ShowDialog() == DialogResult.OK)
        {
            StreamReader sr = new StreamReader(dlg.FileName);
            bindsrc.DataSource = xmlser.Deserialize(sr);
            sr.Close();
        }
    }
    void FileSaveAsOnClick(object objSrc, EventArgs args)
    {
        SaveFileDialog dlg = new SaveFileDialog();
        dlg.Filter = strFilter;

        if (dlg.ShowDialog() == DialogResult.OK)
        {
            StreamWriter sw = new StreamWriter(dlg.FileName);
            xmlser.Serialize(sw, bindsrc.DataSource);
            sw.Close();
        }
    }
}
