//--------------------------------------------------------
// DataGridViewWithBinding.cs (c) 2005 by Charles Petzold
//--------------------------------------------------------
using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Xml.Serialization;

class DataGridViewWithBinding : Form
{
    const string strFilter = "Schuldateien (*.SchoolXml)|" +
                         "*.SchoolXml|Alle Dateien (*.*)|*.*";
    XmlSerializer xmlser = new XmlSerializer(typeof(School));
    BindingSource bindsrc = new BindingSource();

    [STAThread]
    public static void Main()
    {
        Application.EnableVisualStyles();
        Application.Run(new DataGridViewWithBinding());
    }
    public DataGridViewWithBinding()
    {
        Text = "DataGridView mit Bindung";
        Width *= 2;

        // BindingSource initialisieren.
        FileNewOnClick(null, EventArgs.Empty);

        // DataGridView-Steuerelement erstellen.
        DataGridView grid = new DataGridView();
        grid.Parent = this;
        grid.AutoSize = true;
        grid.Dock = DockStyle.Fill;

        // Steuerelement an Datenquelle binden.
        grid.AutoGenerateColumns = false;
        grid.DataSource = bindsrc;
        
        // Gewnschte Spalten fr DataGridView anlegen.
        DataGridViewComboBoxColumn colCombo = new DataGridViewComboBoxColumn();
        colCombo.DataPropertyName = "Courtesy";
        colCombo.DataSource = Enum.GetValues(typeof(CourtesyTitle));
        colCombo.HeaderText = "Anrede";
        grid.Columns.Add(colCombo);

        DataGridViewTextBoxColumn colText = new DataGridViewTextBoxColumn();
        colText.DataPropertyName = "FirstName";
        colText.HeaderText = "Vorname";
        grid.Columns.Add(colText);

        colText = new DataGridViewTextBoxColumn();
        colText.DataPropertyName = "LastName";
        colText.HeaderText = "Nachname";
        grid.Columns.Add(colText);

        CalendarColumn colBirth = new CalendarColumn();
        colBirth.DataPropertyName = "BirthDate";
        colBirth.HeaderText = "Geburtsdatum";
        grid.Columns.Add(colBirth);

        DataGridViewCheckBoxColumn colCheck = new DataGridViewCheckBoxColumn();
        colCheck.DataPropertyName = "Enrolled";
        colCheck.HeaderText = "Registriert?";
        grid.Columns.Add(colCheck);

        // Men erstellen.
        MenuStrip menu = new MenuStrip();
        menu.Parent = this;
        ToolStripMenuItem item = (ToolStripMenuItem)menu.Items.Add("&Datei");
        item.DropDownItems.Add("&Neu", null, FileNewOnClick);
        item.DropDownItems.Add("&ffnen...", null, FileOpenOnClick);
        item.DropDownItems.Add("Speichern &unter...", null, FileSaveAsOnClick);
    }
    void FileNewOnClick(object objSrc, EventArgs args)
    {
        bindsrc.DataSource = new School();
        bindsrc.DataMember = "Students";
    }
    void FileOpenOnClick(object objSrc, EventArgs args)
    {
        OpenFileDialog dlg = new OpenFileDialog();
        dlg.Filter = strFilter;

        if (dlg.ShowDialog() == DialogResult.OK)
        {
            // School-Objekt als XML einlesen.
            StreamReader sr = new StreamReader(dlg.FileName);
            bindsrc.DataSource = xmlser.Deserialize(sr);
            sr.Close();
        }
    }
    void FileSaveAsOnClick(object objSrc, EventArgs args)
    {
        SaveFileDialog dlg = new SaveFileDialog();
        dlg.Filter = strFilter;

        if (dlg.ShowDialog() == DialogResult.OK)
        {
            // School-Objekt als XML schreiben.
            StreamWriter sw = new StreamWriter(dlg.FileName);
            xmlser.Serialize(sw, bindsrc.DataSource);
            sw.Close();
        }
    }
}
