//------------------------------------------
// RgbScroll.cs (c) 2005 by Charles Petzold
//------------------------------------------
using System;
using System.Drawing;
using System.Windows.Forms;

class ColorScrollTable : Form
{
    [STAThread]
    public static void Main()
    {
        Application.EnableVisualStyles();
        Application.Run(new ColorScrollTable());
    }
    public ColorScrollTable()
    {
        Text = "RgbScroll mit TableLayoutPanel";

        // SplitContainer erstellen, der den Clientbereich fllt.
        SplitContainer splt = new SplitContainer();
        splt.Parent = this;
        splt.Dock = DockStyle.Fill;
        splt.SplitterDistance = ClientSize.Width / 2;

        // Das TableLayoutPanel kommt links neben den Splitter.
        TableLayoutPanel table = new TableLayoutPanel();
        table.Parent = splt.Panel1;
        table.Dock = DockStyle.Fill;
        table.BackColor = Color.White;
        table.ColumnCount = 3;
        table.RowCount = 3;

        // Das rechte SplitterPanel zum Anzeigen der Farbe erstellen.
        Panel pnlColor = splt.Panel2;

        // Ein neues Rgb-Objekt erstllen und die Farbe pnlColor zuweisen.
        Rgb rgb = new Rgb();
        rgb.Color = pnlColor.BackColor;

        // Hintergrundfarbe an rgb binden.
        pnlColor.DataBindings.Add("BackColor", rgb, "Color");

        // Array fr Farbnamen.
        string[] astrColors = { "Red", "Green", "Blue" };

        // Die drei Spalten (rot, grn, blau) in einer Schleife durchgehen.
        for (int col = 0; col < 3; col++)
        {
            // Die Beschriftungen oben geben die Farbe an.
            Label lbl = new Label();
            lbl.AutoSize = true;
            lbl.Anchor = AnchorStyles.None;
            lbl.Text = astrColors[col];
            lbl.ForeColor = Color.FromName(astrColors[col]);
            table.Controls.Add(lbl, col, 0);

            // Scrollbar zum Einstellen neuer Werte ist an rgb gebunden.
            VScrollBar vscrl = new VScrollBar();
            vscrl.Parent = table;
            vscrl.Anchor = AnchorStyles.Top | AnchorStyles.Bottom;
            vscrl.TabStop = true;
            vscrl.LargeChange = 16;
            vscrl.Maximum = 255 + vscrl.LargeChange - 1;
            vscrl.DataBindings.Add("Value", rgb, astrColors[col]);
            vscrl.DataBindings[0].DataSourceUpdateMode = 
                                    DataSourceUpdateMode.OnPropertyChanged;
            table.Controls.Add(vscrl, col, 1);

            // Das Beschriftungsfeld fr den Farbwert wird an den Scrollbar gebunden.
            Label lblValue = new Label();
            lblValue.AutoSize = true;
            lblValue.Anchor = AnchorStyles.None;
            lblValue.ForeColor = Color.FromName(astrColors[col]);
            lblValue.DataBindings.Add("Text", vscrl, "Value");
            table.Controls.Add(lblValue, col, 2);

            // ber ColumnStyles drei gleich breite Spalten auswhlen.
            table.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 33));
        }

        // RowStyles legt fest, dass die mittlere Zeile so gro wie mglich ist.
        table.RowStyles.Add(new RowStyle(SizeType.AutoSize));
        table.RowStyles.Add(new RowStyle(SizeType.Percent, 100));
        table.RowStyles.Add(new RowStyle(SizeType.AutoSize));
    }
}