//-------------------------------------------------------------------------
// CalendarEditingControl.cs, from the MSDN topic 
//              "How to: Host Controls in Windows Forms DataGridView Cells
//-------------------------------------------------------------------------
using System;
using System.Windows.Forms;

class CalendarEditingControl : DateTimePicker, IDataGridViewEditingControl
{
    DataGridView dataGridView;
    private bool valueChanged = false;
    int rowIndex;

    public CalendarEditingControl()
    {
        this.Format = DateTimePickerFormat.Short;
    }

    // Implementiert die Eigenschaft 
    // IDataGridViewEditingControl.EditingControlFormattedValue.
    public object EditingControlFormattedValue
    {
        get
        {
            return this.Value.ToShortDateString();
        }
        set
        {
            String newValue = value as String;
            if (newValue != null)
            {
                this.Value = DateTime.Parse(newValue);
            }
        }
    }

    // Implementiert die Methode
    // IDataGridViewEditingControl.GetEditingControlFormattedValue.
    public object GetEditingControlFormattedValue(
        DataGridViewDataErrorContexts context)
    {
        return EditingControlFormattedValue;
    }

    // Implementiert die Methode
    // IDataGridViewEditingControl.ApplyCellStyleToEditingControl.
    public void ApplyCellStyleToEditingControl(
        DataGridViewCellStyle dataGridViewCellStyle)
    {
        this.Font = dataGridViewCellStyle.Font;
        this.CalendarForeColor = dataGridViewCellStyle.ForeColor;
        this.CalendarMonthBackground = dataGridViewCellStyle.BackColor;
    }

    // Implementiert die Eigenschaft
    // IDataGridViewEditingControl.EditingControlRowIndex.
    public int EditingControlRowIndex
    {
        get
        {
            return rowIndex;
        }
        set
        {
            rowIndex = value;
        }
    }

    // Implementiert die Methode
    // IDataGridViewEditingControl.EditingControlWantsInputKey.
    public bool EditingControlWantsInputKey(
        Keys key, bool dataGridViewWantsInputKey)
    {
        // DateTimePicker soll die aufgelisteten Tasten verarbeiten.
        switch (key & Keys.KeyCode)
        {
            case Keys.Left:
            case Keys.Up:
            case Keys.Down:
            case Keys.Right:
            case Keys.Home:
            case Keys.End:
            case Keys.PageDown:
            case Keys.PageUp:
                return true;
            default:
                return false;
        }
    }

    // Implementiert die Methode
    // IDataGridViewEditingControl.PrepareEditingControlForEdit.
    public void PrepareEditingControlForEdit(bool selectAll)
    {
        // Keine Vorbereitungen ntig.
    }

    // Implementiert die Eigenschaft IDataGridViewEditingControl
    // .RepositionEditingControlOnValueChange.
    public bool RepositionEditingControlOnValueChange
    {
        get
        {
            return false;
        }
    }

    // Implementiert die Eigenschaft IDataGridViewEditingControl
    // .EditingControlDataGridView.
    public DataGridView EditingControlDataGridView
    {
        get
        {
            return dataGridView;
        }
        set
        {
            dataGridView = value;
        }
    }

    // Implementiert die Eigenschaft IDataGridViewEditingControl
    // .EditingControlValueChanged.
    public bool EditingControlValueChanged
    {
        get
        {
            return valueChanged;
        }
        set
        {
            valueChanged = value;
        }
    }

    // Implementiert die Eigenschaft IDataGridViewEditingControl
    // .EditingPanelCursor.
    public Cursor EditingPanelCursor
    {
        get
        {
            return base.Cursor;
        }
    }

    protected override void OnValueChanged(EventArgs eventargs)
    {
        // Das DataGridView benachrichtigen, dass sich der Inhalt der
        // Zelle verndert hat.
        valueChanged = true;
        this.EditingControlDataGridView.NotifyCurrentCellDirty(true);
        base.OnValueChanged(eventargs);
    }
}
