//--------------------------------------------------------------
// UnboundDataGridViewWithFileIO.cs (c) 2005 by Charles Petzold
//--------------------------------------------------------------
using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Xml.Serialization;

class UnboundDataGridViewWithFileIO : UnboundDataGridView
{
    const string strFilter = "Schuldateien (*.SchoolXml)|" +
                             "*.SchoolXml|Alle Dateien (*.*)|*.*";
    XmlSerializer xmlser = new XmlSerializer(typeof(School));

    [STAThread]
    public new static void Main()
    {
        Application.EnableVisualStyles();
        Application.Run(new UnboundDataGridViewWithFileIO());
    }
    public UnboundDataGridViewWithFileIO()
    {
        Text = "Ungebundenes DataGridView mit Datei-E/A";

        // Men erstellen.
        MenuStrip menu = new MenuStrip();
        menu.Parent = this;
        ToolStripMenuItem item = (ToolStripMenuItem) menu.Items.Add("&Datei");
        item.DropDownItems.Add("&Neu", null, FileNewOnClick);
        item.DropDownItems.Add("&ffnen...", null, FileOpenOnClick);
        item.DropDownItems.Add("Speichern &unter...", null, FileSaveAsOnClick);
    }
    void FileNewOnClick(object objSrc, EventArgs args)
    {
        grid.Rows.Clear();
    }
    void FileOpenOnClick(object objSrc, EventArgs args)
    {
        OpenFileDialog dlg = new OpenFileDialog();
        dlg.Filter = strFilter;

        if (dlg.ShowDialog() == DialogResult.OK)
        {
            // School-Objekt als XML einlesen.
            StreamReader sr = new StreamReader(dlg.FileName);
            School sch = (School) xmlser.Deserialize(sr);
            sr.Close();

            // Vorhanden Zeilen im DataGridView lschen.
            grid.Rows.Clear();

            // Fr jeden Schler eine Zeile zum DataGridView hinzufgen.
            foreach (Student sdt in sch.Students)
            {
                int index = grid.Rows.Add();

                grid["Courtesy", index].Value = sdt.Courtesy;
                grid["FirstName", index].Value = sdt.FirstName;
                grid["LastName", index].Value = sdt.LastName;
                grid["BirthDate", index].Value = 
                                        sdt.BirthDate.ToShortDateString();
                grid["Enrolled", index].Value = sdt.Enrolled;
            }
        }
    }
    void FileSaveAsOnClick(object objSrc, EventArgs args)
    {
        SaveFileDialog dlg = new SaveFileDialog();
        dlg.Filter = strFilter;

        if (dlg.ShowDialog() == DialogResult.OK)
        {
            // Eventuell aktive Editiervorgnge beenden.
            grid.EndEdit();

            // Neues School-Objekt erstellen.
            School sch = new School();

            // Student-Objekte aus den Zeilen im DataGridView hinzufgen.
            foreach (DataGridViewRow row in grid.Rows)
            {
                if (row.IsNewRow)
                    continue;

                Student sdt = new Student();
                sdt.Courtesy = (CourtesyTitle) row.Cells["Courtesy"].Value;
                sdt.FirstName = (string) row.Cells["FirstName"].Value;
                sdt.LastName = (string)row.Cells["LastName"].Value;
                sdt.BirthDate = 
                        DateTime.Parse((string) row.Cells["BirthDate"].Value);
                sdt.Enrolled = (bool)row.Cells["Enrolled"].Value;

                sch.Students.Add(sdt);
            }

            // School-Objekt als XML schreiben.
            StreamWriter sw = new StreamWriter(dlg.FileName);
            xmlser.Serialize(sw, sch);
            sw.Close();
        }
    }
}