//------------------------------------------------
// ControlExplorer.cs (c) 2005 by Charles Petzold
//------------------------------------------------
using System;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;

[assembly: AssemblyTitle("Control Explorer")]
[assembly: AssemblyDescription("Developer Tool for Windows Forms")]
[assembly: AssemblyCompany("www.charlespetzold.com")]
[assembly: AssemblyProduct("Control Explorer")]
[assembly: AssemblyCopyright("(c) 2005 by Charles Petzold")]
[assembly: AssemblyVersion("1.0.*")]
[assembly: AssemblyFileVersion("1.0.0.0")]

class ControlExplorer: Form
{
    const string strResource = "ControlExplorer";
    Panel pnl;

    [STAThread]
    public static void Main()
    {
        Application.EnableVisualStyles();
        Application.Run(new ControlExplorer());
    }

    public ControlExplorer()
    {
        Text = "Control Explorer";
        Icon = new Icon(GetType(), strResource + ".ControlExplorer.ico");

        pnl = new Panel();
        pnl.Parent = this;
        pnl.Dock = DockStyle.Fill;

        MenuStrip menu = new MenuStrip();
        menu.Parent = this;
        menu.Items.Add(new ControlMenuItem(MenuItemOnClick));

        ToolStripMenuItem itemHelp = new ToolStripMenuItem("&Hilfe");
        menu.Items.Add(itemHelp);
        ToolStripMenuItem itemAbout = new ToolStripMenuItem();
        itemAbout.Text = "&Info ber Control Explorer...";
        itemAbout.Click += AboutOnClick;
        itemHelp.DropDownItems.Add(itemAbout);
    }

    void MenuItemOnClick(object objSrc, EventArgs args)
    {
        // Menelement und zugehrige Klasse ermitteln.
        ToolStripMenuItem item = objSrc as ToolStripMenuItem;
        Type typ = (Type)item.Tag;

        // Alles zum Erstellen eines Objekts dieses Typs vorbereiten.
        ConstructorInfo ci = typ.GetConstructor(System.Type.EmptyTypes);
        Control ctrl;

        // Versuchen, ein Objekt dieses Typs zu erstellen.
        try
        {
            ctrl = (Control)ci.Invoke(null);
        }
        catch (Exception exc)
        {
            MessageBox.Show(exc.Message, Text);
            return;
        }

        // Dialogfeld mit einem PropertyGrid-Steuerlement erstellen.
        PropertiesAndEventsDialog dlg = new PropertiesAndEventsDialog();
        dlg.Owner = this;
        dlg.Text = "Eigenschaften von " + item.Text;
        dlg.SelectedObject = ctrl;
        dlg.Closed += new EventHandler(DialogOnClosed);
        dlg.Show();

        // Falls der Eigenschaft Parent kein Wert zugewiesen werden kann, ist
        //  es wahrscheinlich eine Form. Dann einfach Show aufrufen.
        try
        {
            ctrl.Parent = pnl;
        }
        catch
        {
            ctrl.Show();
        }
    }

    // Beim Schlieen des Eigenschaftendialogfelds das Steuerelement 
    //  beseitigen.
    void DialogOnClosed(object objSrc, EventArgs args)
    {
        PropertiesAndEventsDialog dlg = (PropertiesAndEventsDialog)objSrc;
        Control ctrl = (Control)dlg.SelectedObject;
        ctrl.Dispose();
    }
    void AboutOnClick(object objSrc, EventArgs args)
    {
        new AboutDialog(strResource).ShowDialog(); 
    }
}
