//------------------------------------------------
// ControlMenuItem.cs (c) 2005 by Charles Petzold
// Aus dem Programm ControlExplorer
//------------------------------------------------
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;

class ControlMenuItem : ToolStripMenuItem
{
    public ControlMenuItem(EventHandler evtClick)
    {
        // Die Assembly ermitteln, in der die Klasse Control definiert ist.
        Assembly asbly = Assembly.GetAssembly(typeof(Control));

        // Dies ist ein Array aller Typen in dieser Klasse
        Type[] atype = asbly.GetTypes();

        // Wir speichern die von Control abgeleiteten Klassen in 
        //  einer sortierten Liste.
        SortedList<string, ToolStripMenuItem> sortlst = 
            new SortedList<string, ToolStripMenuItem>();

        Text = "Control";
        Tag = typeof(Control);
        sortlst.Add("Control", this);

        // Alle Typen aus dem Array aufzhlen lassen.
        // Fr Control und davon abgeleitete Klassen Menelemente erstellen und
        //  zum Objekt SortedList hinzufgen.
        // Beachten Sie, wie die Eigenschaft Tag des Menelements mit 
        //  dem Type-Objekt verknpft ist.
        foreach (Type typ in atype)
        {
            if (typ.IsPublic && (typ.IsSubclassOf(typeof(Control))))
            {
                ToolStripMenuItem item = new ToolStripMenuItem(typ.Name);
                item.Click += evtClick;
                item.Tag = typ;
                sortlst.Add(typ.Name, item);
            }
        }

        // Die sortierte Liste durchgehen und die bergeordneten Objekte 
        //  der Menelemente eintragen.
        foreach (KeyValuePair<string, ToolStripMenuItem> kvp in sortlst)
        {
            if (kvp.Key != "Control")
            {
                string strParent = ((Type)kvp.Value.Tag).BaseType.Name;
                ToolStripMenuItem itemParent = sortlst[strParent];
                itemParent.DropDownItems.Add(kvp.Value);

                // itemParent soll keine Ereignishandler haben!
                itemParent.Click -= evtClick;
            }
        }

        // Erneut durchgehen:
        //  Falls abstrakt und auswhlbar, deaktivieren.
        //  Falls nicht abstrakt und nicht auswhlbar, neues Element hinzufgen.
        foreach (KeyValuePair<string, ToolStripMenuItem> kvp in sortlst)
        {
            Type typ = (Type) kvp.Value.Tag;

            if (typ.IsAbstract && kvp.Value.DropDownItems.Count == 0)
                kvp.Value.Enabled = false;

            if (!typ.IsAbstract && kvp.Value.DropDownItems.Count > 0)
            {
                ToolStripMenuItem item = new ToolStripMenuItem(kvp.Value.Text);
                item.Click += evtClick;
                item.Tag = typ;
                kvp.Value.DropDownItems.Insert(0, item);
            }
        }
    }
}