//--------------------------------------------
// EventLogger.cs (c) 2005 by Charles Petzold
// Aus dem Programm ControlExplorer
//--------------------------------------------
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Reflection;
using System.Reflection.Emit;
using System.Windows.Forms;

class EventLogger: SplitContainer
{
    object objSelected;
    CheckBox chkbox;
    CheckedListBox lstbox;
    ConsoleControl cons;
    Dictionary<string, Delegate> deledict = new Dictionary<string, Delegate>();
    static Dictionary<object, ConsoleControl> consdict = 
        new Dictionary<object, ConsoleControl>();

    // ffentliche Eigenschaft fr SelectedObject.
    public object SelectedObject
    {
        get
        {
            return objSelected;
        }
        set
        {
            if (objSelected != null)
                consdict.Remove(objSelected);

            objSelected = value;

            if (objSelected != null)
                consdict.Add(objSelected, cons);

            FullRefresh();
        }
    }
    // Konstruktor.
    public EventLogger()
    {
        // CheckedListBox zeigt alle Ereignisse an, die das 
        //  Steuerelement untersttzt.
        lstbox = new CheckedListBox();
        lstbox.Parent = Panel1;
        lstbox.Dock = DockStyle.Fill;
        lstbox.CheckOnClick = true;
        lstbox.Sorted = true;
        lstbox.ItemCheck += ListBoxOnItemCheck;

        // CheckBox zum Aktivieren/Deaktivieren aller Ereignisse.
        chkbox = new CheckBox();
        chkbox.Parent = Panel1;
        chkbox.AutoSize = true;
        chkbox.AutoCheck = false;
        chkbox.ThreeState = true;
        chkbox.Text = "Alle";
        chkbox.Dock = DockStyle.Top;
        chkbox.Click += CheckBoxOnClick;

        // ConsoleControl zum Protokollieren der Ereignisse.
        cons = new ConsoleControl();
        cons.Parent = Panel2;
        cons.Dock = DockStyle.Fill;
    }
    // ListBoxOnItemCheck: Ereignishandler installieren/entfernen
    void ListBoxOnItemCheck(object objSrc, ItemCheckEventArgs args)
    {
        if (args.CurrentValue == args.NewValue)
            return;

        CheckedListBox lstbox = (CheckedListBox) objSrc;
        string strEvent = lstbox.Items[args.Index].ToString();

        // Handler installieren, falls das Element aktiviert ist.
        if (args.NewValue == CheckState.Checked)
        {
            AttachHandler(strEvent);

            if (lstbox.CheckedIndices.Count + 1 == lstbox.Items.Count)
                chkbox.CheckState = CheckState.Checked;
            else
                chkbox.CheckState = CheckState.Indeterminate;
        }
        // Andernfalls Handler entfernen.
        else
        {
            RemoveHandler(strEvent);

            if (lstbox.CheckedIndices.Count == 1)
                chkbox.CheckState = CheckState.Unchecked;
            else
                chkbox.CheckState = CheckState.Indeterminate;
        }
    }
    // Wenn das Kontrollkstchen "Alle" aktiviert oder deaktiviert wird,
    //  nderungen in die Liste eintragen.
    void CheckBoxOnClick(object objSrc, EventArgs args)
    {
        CheckBox chkbox = (CheckBox)objSrc;

        // ListBoxOnItemChecked soll den Zustand der Hkchen ndern.
        if (chkbox.CheckState == CheckState.Unchecked)
        {
            for (int i = 0; i < lstbox.Items.Count; i++)
                lstbox.SetItemChecked(i, true);
        }
        else
        {
            for (int i = 0; i < lstbox.Items.Count; i++)
                lstbox.SetItemChecked(i, false);
        }
    }
    // FullRefresh wird aufgerufen, wenn sich SelectedProperty ndert.
    void FullRefresh()
    {
        cons.Clear();
        lstbox.Items.Clear();

        if (SelectedObject == null)
            return;

        // Alle Ereignisse in ein Array eintragen.
        EventInfo[] aevtinfo = SelectedObject.GetType().GetEvents();

        lstbox.BeginUpdate();

        // In einer Schleife ber die Ereignisse gehen.
        foreach (EventInfo evtinfo in aevtinfo)
        {
            bool bChecked = false;

            // Ereignis aktivieren, falls SelectedObject ein Steuerelement ist.
            if (SelectedObject.GetType() == typeof(Control))
                bChecked = true;

            // Andernfalls ermitteln, ob das Ereignis nicht von dem
            //  Steuerelement implementiert wird.
            else if (SelectedObject is Control)
                bChecked = typeof(Control).GetEvent(evtinfo.Name) == null;

            lstbox.Items.Add(evtinfo.Name, bChecked);
        }
        lstbox.EndUpdate();
    }
    // AttachHandler
    void AttachHandler(string strEvent)
    {
        // Informationen ber das Ereignis.
        EventInfo evtinfo = SelectedObject.GetType().GetEvent(strEvent);

        // Der Typ des Ereignishandlers.
        Type htype = evtinfo.EventHandlerType;

        // Informationen ber die Ereignishandlermethode.
        MethodInfo[] methinfo = htype.GetMethods();

        // Parameter des Ereignishandlers.
        ParameterInfo[] parminfo = methinfo[0].GetParameters();

        // Eine Methode mit einem Namen wie "ClickEventHandler"
        //  mit dem passenden Rckgabetyp und Parametern erstellen.
        DynamicMethod dynameth = 
            new DynamicMethod(strEvent + "EventHandler", typeof(void),
                              new Type[] { typeof(object), 
                                parminfo[1].ParameterType }, 
                              GetType());

        // Mit ILGenerator kann Code fr diese Methode generiert werden.
        ILGenerator ilg = dynameth.GetILGenerator();

        // Der Ereignishandler ruft die statische Methode EventDump auf.
        MethodInfo miEventDump = GetType().GetMethod("EventDump");

        // Code generieren, der den Ereignisnamen und zwei Argumente 
        //  auf den Stack legt und dann die Methode EventDump aufruft.
        ilg.Emit(OpCodes.Ldstr, strEvent);
        ilg.Emit(OpCodes.Ldarg_0);
        ilg.Emit(OpCodes.Ldarg_1);
        ilg.EmitCall(OpCodes.Call, miEventDump, null);
        ilg.Emit(OpCodes.Ret);

        // Einen Delegaten entsprechend dem Ereignishandler erstellen.
        Delegate dynadele = dynameth.CreateDelegate(htype);

        // Schlielich den Ereignishandler installieren.
        evtinfo.AddEventHandler(SelectedObject, dynadele);

        // Den Delegaten zu einem Dictionary hinzufgen, damit der
        //  Ereignishandler spter wieder entfernt werden kann.
        deledict.Add(strEvent, dynadele);
    }
    // Ereignishandler entfernen.
    void RemoveHandler(string strEvent)
    {
        EventInfo evtinfo = SelectedObject.GetType().GetEvent(strEvent);
        evtinfo.RemoveEventHandler(SelectedObject, deledict[strEvent]);
        deledict.Remove(strEvent);
    }
    // Diese statische Methode zeigt das Ereignis an. 
    // Da die Methode statisch sein muss, muss sie das korrekte 
    //  ConsoleControl-Objekt aus dem ConsoleControl-Dictionary (consdict) abrufen.
    // Der letzte Parameter ist zwar als EventArgs definiert, aber bei vielen
    //  Ereignissen ist es in Wirklichkeit eine davon abgeleitete Klasse.
    public static void EventDump(string strEvent, object objSrc, EventArgs args)
    {
        // Das korrekte ConsoleControl aus dem statischen Dictionary heraussuchen.
        ConsoleControl cons = consdict[objSrc];

        // Das Ereignis mit Sekonden und Millisekunden anzeigen.
        DateTime dt = DateTime.Now;
        cons.Write("{0}.{1:D3} {2}", dt.Second % 10, dt.Millisecond, strEvent);

        // Alle Eigenschaften in EventArgs (oder abgeleiteter Klasse) anzeigen.
        PropertyInfo[] api = args.GetType().GetProperties();

        foreach (PropertyInfo pi in api)
            cons.Write(" {0}={1}", pi.Name, pi.GetValue(args, null));

        // Falls das Ereignis auf "Changed" endet, die neue Eigenschaft anzeigen.
        if (strEvent.EndsWith("Changed"))
        {
            string strProperty = strEvent.Substring(0, strEvent.Length - 7);
            PropertyInfo pi = objSrc.GetType().GetProperty(strProperty);
            cons.Write(" [{0}={1}]", strProperty, pi.GetValue(objSrc, null));
        }
        // Zeile abschlieen.
        cons.WriteLine();
    }
}