//----------------------------------------------------
// MdiBrowser.ViewMenu.cs (c) 2005 by Charles Petzold
// Aus dem Programm MdiBrowser
//----------------------------------------------------
using System;
using System.Drawing;
using System.Windows.Forms;

partial class MdiBrowser : Form
{
    ToolStripMenuItem itemBack, itemForward, itemStop;
    ToolStripMenuItem itemRefresh, itemHome, itemSource;

    ToolStripMenuItem ViewMenu()
    {
        ToolStripMenuItem itemView = new ToolStripMenuItem("&Ansicht");
        itemView.DropDownOpening += ViewOnDropDownOpening;

        ToolStripMenuItem item = new ToolStripMenuItem("&Symbolleiste");
        item.Checked = settings.ViewToolBar;
        item.CheckOnClick = true;
        item.Click += ViewToolBarOnClick;
        itemView.DropDownItems.Add(item);

        item = new ToolStripMenuItem("A&dressleiste");
        item.Checked = settings.ViewAddressBar;
        item.CheckOnClick = true;
        item.Click += ViewAddressBarOnClick;
        itemView.DropDownItems.Add(item);

        itemView.DropDownItems.Add(new ToolStripSeparator());

        itemBack = new ToolStripMenuItem("&Zurck");
        itemBack.Click += BackOnClick;
        itemView.DropDownItems.Add(itemBack);

        itemForward = new ToolStripMenuItem("&Vorwrts");
        itemForward.Click += ForwardOnClick;
        itemView.DropDownItems.Add(itemForward);

        itemStop = new ToolStripMenuItem("&Abbrechen");
        itemStop.Click += StopOnClick;
        itemView.DropDownItems.Add(itemStop);

        itemRefresh = new ToolStripMenuItem("Akt&ualisieren");
        itemRefresh.Click += RefreshOnClick;
        itemView.DropDownItems.Add(itemRefresh);

        itemHome = new ToolStripMenuItem("&Startseite");
        itemHome.Click += HomeOnClick;
        itemView.DropDownItems.Add(itemHome);

        itemView.DropDownItems.Add(new ToolStripSeparator());

        itemSource = new ToolStripMenuItem("&Quelltext");
        itemSource.Click += ViewSourceOnClick;
        itemView.DropDownItems.Add(itemSource);

        return itemView;
    }
    // Ereignishandler fr ffnen des Ansicht-Mens.
    void ViewOnDropDownOpening(object objSrc, EventArgs args)
    {
        BrowserChild bc = ActiveMdiChild as BrowserChild;
        bool bActiveChild = bc != null;

        // Ale diese Elemente deaktivieren, falls kein 
        //  aktives untergeordnetes MDI-Fenster vorhanden.
        itemBack.Enabled = bActiveChild;
        itemForward.Enabled = bActiveChild;
        itemStop.Enabled = bActiveChild;
        itemRefresh.Enabled = bActiveChild;
        itemHome.Enabled = bActiveChild;
        itemSource.Enabled = bActiveChild;

        // Falls es ein aktives Unterfenster gibt, diese Elemente 
        //  abhngig von Eigenschaften aktivieren.
        if (bActiveChild)
        {
            itemBack.Enabled = bc.WebBrowser.CanGoBack;
            itemForward.Enabled = bc.WebBrowser.CanGoForward;
        }
    }
    // Symbolleiste und Adressleiste sichtbar oder unsichtbar machen.
    void ViewToolBarOnClick(object objSrc, EventArgs args)
    {
        ToolStripMenuItem item = objSrc as ToolStripMenuItem;
        tool.Visible = settings.ViewToolBar = item.Checked;
    }
    void ViewAddressBarOnClick(object objSrc, EventArgs args)
    {
        ToolStripMenuItem item = objSrc as ToolStripMenuItem;
        addr.Visible = settings.ViewAddressBar = item.Checked;
    }
    // Implementierung von Zurck, Vorwrts, Abbrechen, Aktualisieren 
    //  und Startseite.
    void BackOnClick(object objSrc, EventArgs args)
    {
        BrowserChild bc = ActiveMdiChild as BrowserChild;

        if (bc != null)
            bc.WebBrowser.GoBack();
    }
    void ForwardOnClick(object objSrc, EventArgs args)
    {
        BrowserChild bc = ActiveMdiChild as BrowserChild;

        if (bc != null)
            bc.WebBrowser.GoForward();
    }
    void StopOnClick(object objSrc, EventArgs args)
    {
        BrowserChild bc = ActiveMdiChild as BrowserChild;

        if (bc != null)
            bc.WebBrowser.Stop();
    }
    void RefreshOnClick(object objSrc, EventArgs args)
    {
        BrowserChild bc = ActiveMdiChild as BrowserChild;

        if (bc != null)
            bc.WebBrowser.Refresh();
    }
    void HomeOnClick(object objSrc, EventArgs args)
    {
        BrowserChild bc = ActiveMdiChild as BrowserChild;

        if (bc != null)
            bc.WebBrowser.Url = new Uri(settings.Home);
    }
    // Menelemente Quelltext.
    void ViewSourceOnClick(object objSrc, EventArgs args)
    {
        BrowserChild bc = ActiveMdiChild as BrowserChild;

        if (bc != null)
        {
            Form frm = new Form();
            frm.Text = bc.WebBrowser.DocumentTitle;

            TextBox txtbox = new TextBox();
            txtbox.Parent = frm;
            txtbox.Multiline = true;
            txtbox.WordWrap = false;
            txtbox.ScrollBars = ScrollBars.Both;
            txtbox.Dock = DockStyle.Fill;
            txtbox.Text = bc.WebBrowser.DocumentText;
            txtbox.Select(0, 0);

            frm.Show();
        }
    }
}
