//-------------------------------------------
// MdiBrowser.cs (c) 2005 by Charles Petzold
// Aus dem Programm MdiBrowser
//-------------------------------------------
using System;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Windows.Forms;

[assembly: AssemblyTitle("MDI Browser")]
[assembly: AssemblyDescription("Multiple Document Interface Web Browser")]
[assembly: AssemblyCompany("www.charlespetzold.com")]
[assembly: AssemblyProduct("MDI Browser")]
[assembly: AssemblyCopyright("(c) 2005 by Charles Petzold")]
[assembly: AssemblyVersion("1.0.*")]
[assembly: AssemblyFileVersion("1.0.0.0")]

partial class MdiBrowser : Form
{
    static string strAppData = Path.Combine(
        Environment.GetFolderPath(
            Environment.SpecialFolder.LocalApplicationData),
                "Petzold\\MdiBrowser\\MdiBrowser.Settings.xml");

    MdiBrowserSettings settings;
    MenuStrip menu;
    ToolStrip addr, tool;

    [STAThread]
    public static void Main()
    {
        Application.EnableVisualStyles();
        Application.Run(new MdiBrowser());
    }
    public MdiBrowser()
    {
        // Grundlegenden Eigenschaften der Form einstellen.
        Text = "MDI-Browser";
        Icon = new Icon(GetType(), "MdiBrowser.MdiBrowser.ico");
        IsMdiContainer = true;

        // Einstellungen laden.
        settings = MdiBrowserSettings.Load(strAppData);

        // Gre und Zustand des Fensters einstellen.
        Bounds = settings.WindowBounds;
        WindowState = settings.WindowState;

        // Adressleisten-ToolStrip erstellen
        addr = CreateAddressBar("MdiBrowser");  // in MdiBrowser.AddrBar.cs

        // Symbolleisten-ToolStrip erstellen
        tool = CreateToolBar("MdiBrowser");  // in MdiBrowser.ToolBar.cs

        // MenuStrip erstellen
        menu = new MenuStrip();
        menu.Parent = this;
        MainMenuStrip = menu;           // Sinnvoll fr MDI.

        menu.Items.Add(FileMenu());     // in MdiBrowser.FileMenu.cs        
        menu.Items.Add(ViewMenu());     // in MdiBrowser.ViewMenu.cs
        menu.Items.Add(FavoritesMenu());// in MdiBrowser.FavoritesMenu.cs
        menu.Items.Add(WindowMenu());   // in MdiBrowser.WindowMenu.cs
        menu.Items.Add(HelpMenu());     // in MdiBrowser.HelpMenu.cs

        // Untergeordnetes Fenster mit der Startseite ffnen.
        Go(settings.Home, false);
    }
    // Diese Methode geht zu einem URL und fgt ihn optional zur Liste hinzu.
    void Go(string strUrl, bool bAddToList)
    {
        BrowserChild bcNew = new BrowserChild();
        bcNew.MdiParent = this;
        bcNew.Icon = Icon;
        bcNew.WebBrowser.Navigate(strUrl);

        // Diese Einstellungen stammen aus dem Objekt settings, wir
        //  aktualisieren sie aber nicht. Dafr wre erforderlich, dass 
        //  wir Ereignishandler fr Resize installieren und die 
        //  letzten Einstellungen des Benutzers speichern.
        bcNew.Bounds = settings.ChildWindowBounds;
        bcNew.WindowState = settings.ChildWindowState;
        bcNew.Show();

        // Falls der URL von Hand eingetippt wurde, zur Liste hinzufgen,
        //  die in den Kombinationsfeldern angezeigt wird.
        if (bAddToList)
        {
            if (settings.ManualUrls.IndexOf(strUrl) == -1)
            {
                settings.ManualUrls.Add(strUrl);
                settings.ManualUrls.Sort();
                settings.Save(strAppData);
            }
        }
    }
    // Einstelllungen speichern, wenn die Form geschlossen wird.
    protected override void OnClosed(EventArgs args)
    {
        settings.WindowState = WindowState;
        settings.WindowBounds = WindowState == 
            FormWindowState.Normal ? Bounds : RestoreBounds;
        settings.Save(strAppData);

        base.OnClosed(args);
    }
}
